"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const debug = require("debug");
const util_1 = require("../util");
const get_data_1 = require("../get-data");
const log = debug('phox:handlers:image');
exports.getImageSibling = (albumsDir, albumName, images, imageIndex, direction) => {
    const siblingIndex = imageIndex + direction;
    if (siblingIndex === -1 || siblingIndex === images.length) {
        return;
    }
    const sibling = images[siblingIndex];
    return {
        title: sibling.meta.title || '',
        linkProps: util_1.getDetailLinkProps(albumsDir, albumName, sibling.filePath),
    };
};
exports.getImageApiData = (config, albumName, imageName) => __awaiter(this, void 0, void 0, function* () {
    const albumDir = path.join(config.contentDir, config.albumsDir, albumName);
    const album = yield get_data_1.getPageContent(`${albumDir}/index.md`, `${config.albumsDir}/`);
    if (!album) {
        return;
    }
    const images = yield get_data_1.getImages(config.albumsDir, albumName);
    const image = images.find(({ fileName }) => fileName.split('.').shift() === imageName);
    if (!image) {
        return;
    }
    const imageIndex = images.indexOf(image);
    const prev = exports.getImageSibling(config.albumsDir, albumName, images, imageIndex, -1);
    const next = exports.getImageSibling(config.albumsDir, albumName, images, imageIndex, 1);
    const back = {
        title: album && album.meta.title,
        linkProps: util_1.getAlbumLinkProps(config.albumsDir, album.name),
    };
    return { image, next, prev, back };
});
exports.default = (config) => (req, res) => __awaiter(this, void 0, void 0, function* () {
    log('Render image "%s" for request %O', req.path, req.params);
    const imageApiData = yield exports.getImageApiData(config, req.params.album, req.params.image);
    if (!imageApiData) {
        res.status(404).json({ message: 'Not found' });
        return;
    }
    res.json(imageApiData);
});
