"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const next = require("next");
const config_1 = require("./config");
const get_data_1 = require("./get-data");
const common_1 = require("./handlers/common");
const page_1 = require("./handlers/page");
const page_data_1 = require("./handlers/page-data");
const frontpage_data_1 = require("./handlers/frontpage-data");
const album_data_1 = require("./handlers/album-data");
const tag_data_1 = require("./handlers/tag-data");
const image_data_1 = require("./handlers/image-data");
const debug = require("debug");
const log = debug('phox:server');
log('Starting phox server.');
exports.default = (router) => __awaiter(this, void 0, void 0, function* () {
    const config = config_1.default();
    const quiet = true;
    const dev = process.env.NODE_ENV !== 'production';
    const app = next({ dev, quiet });
    const handle = app.getRequestHandler();
    get_data_1.initCachePurger(config);
    yield Promise.all([app.prepare(), get_data_1.default(config)]);
    const server = express();
    log('Seting up routes.');
    server.get('/', common_1.default(app, '/index'));
    server.get(`/${config.albumsDir}/`, (req, res) => res.redirect(301, '/'));
    server.get(`/${config.albumsDir}/:album/`, common_1.default(app, '/album'));
    server.get(`/${config.albumsDir}/:album/:image/`, common_1.default(app, '/image'));
    server.get('/tag/:tag/', common_1.default(app, '/tag'));
    server.get('/:page/', page_1.default(config, app, '/default'));
    server.get('/data/index.json', frontpage_data_1.default(config));
    server.get(`/data/${config.albumsDir}/(:album).json`, album_data_1.default(config));
    server.get(`/data/${config.albumsDir}/(:album)/(:image).json`, image_data_1.default(config));
    server.get('/data/tag/(:tag).json', tag_data_1.default);
    server.get('/data/(:page).json', page_data_1.default(config));
    if (router) {
        server.use('/', router);
    }
    // tslint:disable-next-line no-unnecessary-callback-wrapper
    server.get('*', (req, res) => __awaiter(this, void 0, void 0, function* () { return handle(req, res); }));
    return { server, app };
});
