"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const fs_1 = require("fs");
const path = require("path");
const mkdirp = require("mkdirp");
const debug = require("debug");
const get_data_1 = require("./get-data");
const image_data_1 = require("./handlers/image-data");
const util_2 = require("./util");
const log = debug('phox:write-api-data');
const pWriteFile = util_1.promisify(fs_1.writeFile);
const pMkdirp = util_1.promisify(mkdirp);
const stripFileSuffix = (filePath) => filePath.split('.').shift();
const createFolder = (folder) => __awaiter(this, void 0, void 0, function* () {
    yield pMkdirp(folder);
});
const writeData = (destination, dataItem) => __awaiter(this, void 0, void 0, function* () {
    try {
        yield pWriteFile(destination, JSON.stringify(dataItem), 'utf8');
    }
    catch (err) {
        log('Error: %s', err.message || err);
        throw err;
    }
});
const writeFrontpageData = (outDir, data) => __awaiter(this, void 0, void 0, function* () {
    yield writeData(path.join(outDir, 'data/index.json'), data);
});
const writePagesData = (outDir) => (page) => __awaiter(this, void 0, void 0, function* () {
    yield writeData(path.join(outDir, 'data', `${page.name}.json`), page);
});
const writeImageData = (config, album, images, albumFileName) => (image) => __awaiter(this, void 0, void 0, function* () {
    const destination = path.join(stripFileSuffix(albumFileName), `${stripFileSuffix(image.fileName)}.json`);
    const imageIndex = images.indexOf(image);
    const prev = image_data_1.getImageSibling(config.albumsDir, album.content.name, images, imageIndex, -1);
    const next = image_data_1.getImageSibling(config.albumsDir, album.content.name, images, imageIndex, 1);
    const back = {
        title: album.content.meta.title,
        linkProps: util_2.getAlbumLinkProps(config.albumsDir, album.content.name),
    };
    const data = { image, next, prev, back };
    yield writeData(destination, data);
});
const writeAlbumsData = (config) => (album) => __awaiter(this, void 0, void 0, function* () {
    const destination = path.join(config.outDir, 'data', config.albumsDir, `${album.content.name}.json`);
    yield writeData(destination, album);
    yield Promise.all(album.images.map(writeImageData(config, album, album.images, destination)));
});
const writeTagsData = (tagFolder, tags) => __awaiter(this, void 0, void 0, function* () {
    return Promise.all(tags.map((tag) => __awaiter(this, void 0, void 0, function* () {
        const destination = path.join(tagFolder, `${tag.slug}.json`);
        yield writeData(destination, tag);
    })));
});
exports.default = (config) => __awaiter(this, void 0, void 0, function* () {
    log('Write API data.');
    const { albums, pages, tags } = yield get_data_1.default(config);
    const albumFolders = albums.map(({ content }) => path.join(config.outDir, 'data', config.albumsDir, content.name));
    const tagFolder = path.join(config.outDir, 'data', 'tag');
    const frontpageApiData = yield get_data_1.getFrontpageApiData(config);
    yield pMkdirp(tagFolder);
    yield Promise.all(albumFolders.map(createFolder));
    yield Promise.all([
        writeFrontpageData(config.outDir, frontpageApiData),
        Promise.all(pages
            .filter(({ name }) => name !== 'frontpage')
            .map(writePagesData(config.outDir))),
        Promise.all(albums.map(writeAlbumsData(config))),
        writeTagsData(tagFolder, tags),
    ]);
});
